/*
 * Range.hpp
 *
 *  Created on: Jul 22, 2010
 *      Author: crueger
 */

#ifndef RANGE_HPP_
#define RANGE_HPP_

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iosfwd>

#include "LinearAlgebra/Vector.hpp"

template <class T>
struct range {
  range(const T&,const T&);
  bool isInRange(const T&) const;
  bool isBefore(const T&) const;
  bool isBeyond(const T&) const;

  T first;
  T last;
};

template <class T>
inline range<T>::range(const T &_first,const T &_last) :
first(_first),last(_last)
{}

template <class T>
inline bool range<T>::isInRange(const T &value) const{
  return first <= value && value < last;
}

template <class T>
inline bool range<T>::isBefore(const T &value) const{
  return value < first;
}

template <class T>
inline bool range<T>::isBeyond(const T &value) const{
 return last <= value;
}

template <class T>
inline range<T> makeRange(const T&first, const T&last){
  return range<T>(first,last);
}

template <class T>
inline bool operator<(const range <T> &x, const range<T> &y){
  return (x.first!=y.first)?x.first<y.first:x.last<y.last;
}

template <class T>
inline bool operator==(const range<T> &x,const range<T> &y){
  return x.first==y.first && x.last==y.last;
}

template <class T>
inline bool operator!= (const range<T>& x, const range<T>& y) { return !(x==y); }
template <class T>
inline bool operator>  (const range<T>& x, const range<T>& y) { return y<x; }
template <class T>
inline bool operator<=(const range<T>& x, const range<T>& y) { return !(y<x); }
template <class T>
inline bool operator>= (const range<T>& x, const range<T>& y) { return !(x<y); }
template <class T>
std::ostream &operator<< (std::ostream &ost, const range<T>& y)
{
  ost << "[" << y.first << ";" << y.last << "]";
  return ost;
}


/******************************** Vector ********************************/
/*       template specialization for Vector                             */
/************************************************************************/

class Vector;


template <>
inline bool range<Vector>::isInRange(const Vector &value) const{
  return first[0] <= value[0] && value[0] < last[0] &&
         first[1] <= value[1] && value[1] < last[1] &&
         first[2] <= value[2] && value[2] < last[2];
}

template <>
inline bool range<Vector>::isBefore(const Vector &value) const{
  return value[0] < first[0] || value[1] < first[1] || value[2] < first[2];
}

template <>
inline bool range<Vector>::isBeyond(const Vector &value) const{
 return last[0] <= value[0] || last[1] <= value[1] || last[2] <= value[2];
}

#endif /* RANGE_HPP_ */
